@@ -32,7 +32,7 @@ extern "C" {
 static xlibparam xparams = { 0, 0, 0 };
 static xdisplay *d;
 
-static int lowColor;
+static bool lowColor, paletteInstalled;
 static unsigned char mapping[64];
 
 int setPalette(int i,int r,int g,int b) {
@@ -50,11 +50,10 @@ void screenInit(int xHint,int yHint,int widthHint,int heightHint) {
 
   #define BOUND(x) ((x) > 255 ? 255 : (x))
   #define PEAKIFY(x) BOUND((x) - (x)*(255-(x))/255/2)
-  int i;
 
   lowColor = (d->depth <= 8);
 
-  if (!lowColor) { 
+/*  if (!lowColor) { 
     for(i=0;i<256;i++)
       attempt(setPalette(i,PEAKIFY((i&15*16)),
                    PEAKIFY((i&15)*16+(i&15*16)/4),
@@ -66,11 +65,34 @@ void screenInit(int xHint,int yHint,int widthHint,int heightHint) {
                                         PEAKIFY((i&7)*32+(i&7*8)*2),
                                         PEAKIFY((i&7)*32)),
         " in X: could not allocate sufficient palette entries");
+  }*/
+}
+
+void screenSetPalette(unsigned char *palette) {
+  int i;
+
+  if (paletteInstalled)
+    xfree_colors(d);
+
+  if (!lowColor) {
+    for(i=0;i<256;i++)
+      attempt(setPalette(i,palette[i*3],palette[i*3+1],palette[i*3+2]),
+              " in X: could not allocate sufficient palette entries");
+  } else {
+    const int array[8] = {0,2,5,7,9,11,13,15};
+    for(i=0;i<64;i++) {
+      int p = (array[i&7]+array[i/8]*16) *3;
+      attempt(mapping[i] = setPalette(i,palette[p],palette[p+1],palette[p+2]),
+        " in X: could not allocate sufficient palette entries");
+    }
   }
+  
+  paletteInstalled = true;
 }
 
 void screenEnd() {
-  xfree_colors(d);
+  if (paletteInstalled)
+    xfree_colors(d);
   xfree_display(d);
 }
 
@@ -79,23 +101,17 @@ int sizeUpdate() {
   if (xsize_update(d,&newWidth,&newHeight)) {
     if (newWidth == outWidth && newHeight == outHeight)
       return 0;
-    delete[] output;
-    outWidth = newWidth;
-    outHeight = newHeight;
-    output = new unsigned char [outWidth*outHeight*2];
-    memset(output,32,outWidth*outHeight*2);
+    //delete[] output;
+    //outWidth = newWidth;
+    //outHeight = newHeight;
+    //output = new unsigned char [outWidth*outHeight*2];
+    //memset(output,32,outWidth*outHeight*2);
+    allocOutput(newWidth,newHeight);
     return 1;
   }
   return 0;
 }
 
-void sizeRequest(int width,int height) {
-  if (width != outWidth || height != outHeight) {
-    xsize_set(d,width,height);
-    XFlush(d->display);
-  }
-}
-
 void inputUpdate(int &mouseX,int &mouseY,int &mouseButtons,char &keyHit) {
   xmouse_update(d);
   mouseX = xmouse_x(d);
@@ -113,14 +129,23 @@ void screenShow(void) {
       register unsigned int const r1 = *(ptr2++);
       register unsigned int const r2 = *(ptr2++);
     
-      if (r1 || r2) {
+      //if (r1 || r2) {
+#ifdef LITTLEENDIAN
         register unsigned int const v = 
              mapping[((r1&0xe0ul)>>5)|((r1&0xe000ul)>>10)]
             |mapping[((r1&0xe00000ul)>>21)|((r1&0xe0000000ul)>>26)]*256U; 
         *(ptr1++) = v | 
              mapping[((r2&0xe0ul)>>5)|((r2&0xe000ul)>>10)]*65536U
             |mapping[((r2&0xe00000ul)>>21)|((r2&0xe0000000ul)>>26)]*16777216U; 
-      } else ptr1++;
+#else
+        register unsigned int const v = 
+             mapping[((r2&0xe0ul)>>5)|((r2&0xe000ul)>>10)]
+            |mapping[((r2&0xe00000ul)>>21)|((r2&0xe0000000ul)>>26)]*256U; 
+        *(ptr1++) = v | 
+             mapping[((r1&0xe0ul)>>5)|((r1&0xe000ul)>>10)]*65536U
+            |mapping[((r1&0xe00000ul)>>21)|((r1&0xe0000000ul)>>26)]*16777216U; 
+#endif
+      //} else ptr1++;
     } while (--i); 
   else
     do {
@@ -129,18 +154,31 @@ void screenShow(void) {
       register unsigned int const r1 = *(ptr2++);
       register unsigned int const r2 = *(ptr2++);
     
-      if (r1 || r2) {
+      //if (r1 || r2) {
+#ifdef LITTLEENDIAN
         register unsigned int const v = 
             ((r1 & 0x000000f0ul) >> 4)
           | ((r1 & 0x0000f000ul) >> 8)
           | ((r1 & 0x00f00000ul) >> 12)
           | ((r1 & 0xf0000000ul) >> 16);
         *(ptr1++) = v | 
-          ( ((r2 & 0x000000f0ul) << 12)
-          | ((r2 & 0x0000f000ul) << 8)
-          | ((r2 & 0x00f00000ul) << 4)
-          | ((r2 & 0xf0000000ul)));
-      } else ptr1++;
+            ((r2 & 0x000000f0ul) << 16 -4)
+          | ((r2 & 0x0000f000ul) << 16 -8)
+          | ((r2 & 0x00f00000ul) << 16 -12)
+          | ((r2 & 0xf0000000ul) << 16 -16);
+#else
+        register unsigned int const v = 
+            ((r2 & 0x000000f0ul) >> 4)
+          | ((r2 & 0x0000f000ul) >> 8)
+          | ((r2 & 0x00f00000ul) >> 12)
+          | ((r2 & 0xf0000000ul) >> 16);
+        *(ptr1++) = v | 
+            ((r1 & 0x000000f0ul) << 16 -4)
+          | ((r1 & 0x0000f000ul) << 16 -8)
+          | ((r1 & 0x00f00000ul) << 16 -12)
+          | ((r1 & 0xf0000000ul) << 16 -16);
+#endif
+      //} else ptr1++;
     } while (--i); 
   
   xflip_buffers(d);
